# -*- coding: utf-8 -*-
# -----------------------------------------------------------------------------
# Copyright (c) 2016-2017 Anaconda, Inc.
#
# May be copied and distributed freely only as part of an Anaconda or
# Miniconda installation.
# -----------------------------------------------------------------------------
"""
Tests for splash screens.
"""

# Third party imports
from qtpy.QtCore import Qt  # analysis:ignore
import pytestqt.qtbot as qtbot  # analysis:ignore

# Local imports
from anaconda_navigator.widgets.dialogs.splash import SplashScreen


def test_splash_screen(qtbot):  # analysis:ignore
    widget = SplashScreen()
    qtbot.addWidget(widget)
    test_message = 'Test message'
    widget.show_message(test_message)
    assert widget.get_message() == test_message
